
class fft {
    public:

    #define DIVISIONS 24
    #define VARIANCE 0
    #define SCALE_VALUE_BASE 3000
    #define SCALE_VALUE_MUL 2500
    #define START 5
    #define FINAL_SCALE 0.333
    #define MAX_SIZE 5000

    int freq_group[16];
    float *real;
    float *imag;
    float *real2;
    float *imag2;

    fft()
    {
        memset(freq_group,0,16);

        real = new float[MAX_SIZE];
        imag = new float[MAX_SIZE];
        real2 = new float[MAX_SIZE];
        imag2 = new float[MAX_SIZE];

        }
    ~fft()
    {
            // Free inputs
        delete [] real;real=0;
        delete [] imag;imag=0;

        delete [] real2; real2 = 0;
        delete [] imag2; imag2 = 0;

    }
    unsigned NumberOfBitsNeeded ( unsigned PowerOfTwo )
    {
        unsigned i;

        for ( i=0; ; i++ )
        {
            if ( PowerOfTwo & (1 << i) )
                return i;
        }
    }

    unsigned ReverseBits ( unsigned index, unsigned NumBits )
    {
        unsigned i, rev;

        for ( i=rev=0; i < NumBits; i++ )
        {
            rev = (rev << 1) | (index & 1);
            index >>= 1;
        }

        return rev;
    }

    double Index_to_frequency ( unsigned NumSamples, unsigned Index )
    {
        if ( Index >= NumSamples )
            return 0.0;
        else if ( Index <= NumSamples/2 )
            return (double)Index / (double)NumSamples;

        return -(double)(NumSamples-Index) / (double)NumSamples;
    }

    void fft_float (
        unsigned  NumSamples,
        int       InverseTransform,
        float    *RealIn,
        float    *ImagIn,
        float    *RealOut,
        float    *ImagOut )
    {
        unsigned NumBits;    /* Number of bits needed to store indices */
        unsigned i, j, k, n;
        unsigned BlockSize, BlockEnd;

        double angle_numerator = 2.0 * (3.14159265359);
        double tr, ti;     /* temp real, temp imag */

        if ( InverseTransform )
            angle_numerator = -angle_numerator;

        NumBits = NumberOfBitsNeeded ( NumSamples );

        /*
        **   Do simultaneous data copy and bit-reversal ordering into outputs...
        */

        for ( i=0; i < NumSamples; i++ )
        {
            j = ReverseBits ( i, NumBits );
            RealOut[j] = RealIn[i];
            ImagOut[j] = (ImagIn == NULL) ? 0.0 : ImagIn[i];
        }

        /*
        **   Do the FFT itself...
        */

        BlockEnd = 1;
        for ( BlockSize = 2; BlockSize <= NumSamples; BlockSize <<= 1 )
        {
            double delta_angle = angle_numerator / (double)BlockSize;
            double sm2 = sin ( -2 * delta_angle );
            double sm1 = sin ( -delta_angle );
            double cm2 = cos ( -2 * delta_angle );
            double cm1 = cos ( -delta_angle );
            double w = 2 * cm1;
            double ar[3], ai[3];

            for ( i=0; i < NumSamples; i += BlockSize )
            {
                ar[2] = cm2;
                ar[1] = cm1;

                ai[2] = sm2;
                ai[1] = sm1;

                for ( j=i, n=0; n < BlockEnd; j++, n++ )
                {
                    ar[0] = w*ar[1] - ar[2];
                    ar[2] = ar[1];
                    ar[1] = ar[0];

                    ai[0] = w*ai[1] - ai[2];
                    ai[2] = ai[1];
                    ai[1] = ai[0];

                    k = j + BlockEnd;
                    tr = ar[0]*RealOut[k] - ai[0]*ImagOut[k];
                    ti = ar[0]*ImagOut[k] + ai[0]*RealOut[k];

                    RealOut[k] = RealOut[j] - tr;
                    ImagOut[k] = ImagOut[j] - ti;

                    RealOut[j] += tr;
                    ImagOut[j] += ti;
                }
            }

            BlockEnd = BlockSize;
        }

        /*
        **   Need to normalize if inverse transform...
        */

        if ( InverseTransform )
        {
            double denom = (double)NumSamples;

            for ( i=0; i < NumSamples; i++ )
            {
                RealOut[i] /= denom;
                ImagOut[i] /= denom;
            }
        }
    }

    void do_fft(BYTE* in, int max)
    {
        if(max >= MAX_SIZE/2) max = MAX_SIZE/2;

        // Convert from stereo to mono and set to real, setting imaginary to blank
        for(int i = 0; i < max; i++)
        {
            // Average and normalized
            real[i] = (float)(in[i]);//((float)(in[i << 1]) + (float)(in[(i << 1) + 1])) / (float)(32767 * 2);
            imag[i] = 0.0;
        }

        // Do fft

        fft_float(max, 0, real, imag, real2, imag2);

         // Ensure we have the right count
        int index = 0;

        // Combine into 16 groups
        for(int j = 0; j < DIVISIONS; j++)
        {
            // Smoothing
            int numSamples = ((2 * VARIANCE * index) / max) + (max / (DIVISIONS * 2)) - (VARIANCE / 2);
            if (numSamples < 1) return;

            // Remember before average
            int sampleTotal = 0;

            for(int i = 0; i < numSamples; i++)
            {
                // z = sqrt(real + imaginary)
                sampleTotal += (int)(sqrt((real2[index] * real2[index]) + (imag2[index] * imag2[index])) * FINAL_SCALE * (float)(SCALE_VALUE_BASE + (j * SCALE_VALUE_MUL)));

                // Move to next index
                index++;
            }

            // Place
            int k = j - START;
            if(k >= 0 && k < 16)
            {
                // Divide sample by number of samples to get average and store
                freq_group[k] = sampleTotal / numSamples;

                // Sanity check
                if(freq_group[k] < 0) { freq_group[k] = 0; }
                if(freq_group[k] > 32767) { freq_group[k] = 32767; }
            }

        }

    }
};
