#ifndef _GUI_H_
#define _GUI_H_

#define MAX_GENRE (15)
const char genres[MAX_GENRE][10] = {
  "all",
  "dance","trance","techno","pop",
  "rock","rap","90s","80s","70s","60s",
  "50s","metal","classical","indie"
};



enum _button_states {
    B_OUT = 0,
    B_OVER,
    B_CLICK
};

#define X_OFFSET 8 // The pointer's hotspot is 8px from the left

//simple button class
class gui_button {
    public:

    int s_x;
    int s_y;
    int s_h;
    int s_w;
    long color_out;
    long color_over;
    long color_click;
    long text_color;
    char* text_l1;
    char* text_l2;
    int pad_x;
    int pad_y;
    int btn_state;
    SDL_Surface*    bnt_out;
    SDL_Surface*    bnt_over;
    SDL_Surface*    bnt_down;
    bool            use_images;
    fonts*          fnts;

    gui_button(fonts* f,int x,int y,int w,int h,long c1,long c2,char* t,long tc) :
    btn_state(B_OUT), fnts(f), s_x(x), s_y(y), s_w(w), s_h(h), color_out(c1), color_over(c2),
    text_l1(t), text_color(tc), pad_x(30), pad_y(15),color_click(0),bnt_out(0),bnt_over(0), use_images(false) {}

    gui_button()
    {
        if (bnt_out) SDL_FreeSurface(bnt_out);
        if (bnt_over) SDL_FreeSurface(bnt_over);
        if (bnt_down) SDL_FreeSurface(bnt_down);

    };

    void set_text(char* t)
    {
        if(!t) return;
        else text_l1 = t;

    };

    void set_images(char* over, char* out,char* down)
    {
        bnt_out = SDL_LoadBMP(make_path(out));
        SDL_SetColorKey(bnt_out,SDL_SRCCOLORKEY, SDL_MapRGB(bnt_out->format,0,255,0));

        bnt_over = SDL_LoadBMP(make_path(over));
        SDL_SetColorKey(bnt_over,SDL_SRCCOLORKEY, SDL_MapRGB(bnt_over->format,0,255,0));

        bnt_down = SDL_LoadBMP(make_path(down));
        SDL_SetColorKey(bnt_down,SDL_SRCCOLORKEY, SDL_MapRGB(bnt_down->format, 0,255,0));

        use_images = true;
    };
    void draw()
    {

        if (use_images) {
            if(bnt_out && btn_state == B_OUT){
                SDL_Rect ds = {s_x,s_y,bnt_out->w,bnt_out->h};
                SDL_BlitSurface( bnt_out,0, screen,&ds );
            }else if (bnt_over && btn_state == B_OVER){
                SDL_Rect ds = {s_x,s_y,bnt_over->w,bnt_over->h};
                SDL_BlitSurface( bnt_over,0, screen,&ds );
            }else if (bnt_down && btn_state == B_CLICK){
                SDL_Rect ds = {s_x,s_y,bnt_down->w,bnt_down->h};
                SDL_BlitSurface( bnt_down,0, screen,&ds );
            }
        }else{
            long clr;

            if (btn_state == B_OUT) clr = color_out;
            else if (btn_state == B_OVER) clr = color_over;
            else if (btn_state == B_CLICK) clr = color_click;

            draw_rect(s_x,s_y,s_w,s_h,clr);
        }

        if (text_l1) {
            fnts->change_color(text_color);
            fnts->text(text_l1,s_x+pad_x,s_y+pad_y);
        }

    };

    int hit_test(int x, int y, int event)
    {
        x+=X_OFFSET;
        if (x > s_x && x < (s_x+s_w) && y > s_y && y < (s_y+s_h))
            event == SDL_MOUSEBUTTONDOWN ? btn_state = B_CLICK : btn_state = B_OVER;
        else btn_state = B_OUT;

        return btn_state;
    };
};

class gui {
    public:

    #define MAX_BUTTONS (8)

    fonts*          fnts;

    gui_button*     btns_stations[MAX_BUTTONS];
    gui_button*     genre_change;
    gui_button*     genre_search;
    gui_button*     btn_more_stations;
    gui_button*     btn_prev_stations;

    int             select_pos;
    int             row_height;
    int             start_y;
    int             peakResults[16];

    SDL_Surface*    title;
    SDL_Surface*    player;
    SDL_Surface*    cursor;

    int             genre_selected;
    gui(fonts* f) : fnts(f), select_pos(0), row_height(30), start_y(70), genre_selected(0) {

        memset(peakResults,0,16);

        cursor = SDL_LoadBMP(make_path("cursors/hand.bmp"));
        SDL_SetColorKey(cursor,SDL_SRCCOLORKEY, SDL_MapRGB(cursor->format, 0,0,0));

        title = SDL_LoadBMP(make_path("imgs/title.bmp"));
        SDL_SetColorKey(title,SDL_SRCCOLORKEY, SDL_MapRGB(title->format, 0,255,0));

        player = SDL_LoadBMP(make_path("imgs/player.bmp"));
        SDL_SetColorKey(player,SDL_SRCCOLORKEY, SDL_MapRGB(player->format, 0,255,0));


        // genre change
        genre_change = new gui_button(f,440,84,20,20,0,0,NULL,0);
        genre_change->set_images("imgs/round2.bmp","imgs/round1.bmp","imgs/round3.bmp");

        //search
        genre_search = new gui_button(f,540,84,55,20,0,0,NULL,0);
        genre_search->set_images("imgs/next_over.bmp","imgs/next_out.bmp","imgs/next_down.bmp");

        //more
        btn_more_stations = new gui_button(f,385,355,34,20,0,0,NULL,0);
        btn_more_stations->set_images("imgs/next_over.bmp","imgs/next_out.bmp","imgs/next_down.bmp");

        //back
        btn_prev_stations = new gui_button(f,348,355,34,20,0,0,NULL,0);
        btn_prev_stations->set_images("imgs/previous_over.bmp","imgs/previous_out.bmp","imgs/previous_down.bmp");


        // stations
        loopi(MAX_BUTTONS) {
             btns_stations[i] = new gui_button(f,10,65+i*36,415,36,i%2 ? 0x958BDD : 0x8175D7,0x4754CB,NULL,0x3c1080);
             btns_stations[i]->set_images("imgs/button2.bmp","imgs/button1.bmp","imgs/button3.bmp");
        }


    };

    ~gui(){

        SDL_FreeSurface(cursor);
        SDL_FreeSurface(player);
        SDL_FreeSurface(title);

        if(btn_prev_stations) {
            delete btn_prev_stations;
            btn_prev_stations = 0;
        }

        if(btn_more_stations) {
            delete btn_more_stations;
            btn_more_stations = 0;
        }

        if (genre_search) {
            delete genre_search;
            genre_search = 0;
        }

        if (genre_change) {
            delete genre_change;
            genre_change = 0;
        }

        loopi(MAX_BUTTONS) {
            delete btns_stations[i];
            btns_stations[i]=0;
        };

    };

    int handle_events(SDL_Event* events)
    {
        if( events->type == SDL_MOUSEMOTION
            || events->type == SDL_MOUSEBUTTONDOWN
            || events->type == SDL_MOUSEBUTTONUP
            )
        {
            loopi(MAX_BUTTONS) {
                if (btns_stations[i]->hit_test(events->motion.x,events->motion.y,events->type)==B_CLICK) {
                   connect_to_stream(i);
                }
            }

            if (genre_change->hit_test(events->motion.x,events->motion.y,events->type)==B_CLICK) {
                genre_selected - 1 < 0 ? genre_selected=MAX_GENRE-1 : genre_selected--;
            }

            if (genre_search->hit_test(events->motion.x,events->motion.y,events->type)==B_CLICK) {
                search_genre((char*)genres[genre_selected]);
            }

             if (btn_more_stations->hit_test(events->motion.x,events->motion.y,events->type)==B_CLICK) {
                genre_nex_prev(true,(char*)genres[genre_selected]);
             }

            if(btn_prev_stations->hit_test(events->motion.x,events->motion.y,events->type)==B_CLICK) {
                if (genre_idx>1) genre_nex_prev(false,(char*)genres[genre_selected]);
            }

        }

        return -1;

    };


    void draw(station_list* current_list)
    {
        // title
        SDL_Rect r = { 20,15,title->w,title->h };
        SDL_BlitSurface( title,0, screen,&r);
        // player
        SDL_Rect p = { 430,65,player->w,player->h };
        SDL_BlitSurface( player,0, screen,&p);

        //more stations (next)
        btn_more_stations->draw();
        if (genre_idx>1) btn_prev_stations->draw();

        // genre next/prev Button
        genre_change->draw();
        fnts->change_color(0x3c1080);
        fnts->text(genres[genre_selected],465,89);
        genre_search->draw();


        // loop through stations
        station_list* csl = current_list;//scb->sl_first;
        int i = 0;

        while(csl)
        {
            btns_stations[i]->set_text(csl->station_name);
            btns_stations[i]->draw();
            i++;

            csl = csl->nextnode;
        }


    };

    void draw_cursor(int x,int y)
    {
        SDL_Rect r = { x,y,cursor->w,cursor->h };
        SDL_BlitSurface( cursor,0, screen,&r);
    };

    void draw_visual(int* freq, int freq_len)
    {
        int x = 450;
        int y = 150;
        int bar_height = 50;
        double value = 0.0;

        for (int i=0;i<freq_len;i++)
        {

                // Drop
            peakResults[i] -= (32767 / bar_height);

            // Make sure we don't go too far down
            if(peakResults[i] < 0) { peakResults[i] = 0; }

            // Raise if pushed
            if(peakResults[i] < freq[i])  peakResults[i] = freq[i];


            double percent = (double)peakResults[i] / (double)32767;
            int newY = bar_height - (int)(percent * (double)bar_height);

            draw_rect(x+i*8,y-newY,8,newY,0x312680 + i*5);

        }
    }

    void draw_player(icy* ic)
    {
        char s_status[25] = {0};
        char d_status[100] = {0};
        fnts->change_color(0xcccccc);
        switch(status){
            case 0: strcpy(s_status,"Stopped");
            break;
            case 1: strcpy(s_status,"Playing");
            break;
            case 2: strcpy(s_status,"Buffering");
            break;
            case 3: strcpy(s_status,"Failed to connect");
            break;
        }
        sprintf(d_status,"Status: %s",s_status);
        fnts->change_color(0x3c1080);
        fnts->text(d_status,20,370);


        if (status != STOPPED && status != FAILED)
        {
            char song[55] = {0};
            if (strlen(ic->track_title) >47) {
                memcpy(song,ic->track_title,47);
                strcat(song,"...");
            }else{
                strcpy(song,ic->track_title);
            }

            fnts->text(song,20,380);
        }

        //version name / number
        char version[100] = {0};
        sprintf(version,"%s (%1.1f)",VERSION_NAME,VERSION_NUMBER);
        fnts->text(version,250,50);

    };

};
#endif//_GUI_H_
