#ifndef _ICY_H_
#define _ICY_H_

#define ICY_META_MAX (9)
char icy_meta[ICY_META_MAX][20] = {   "icy-notice1:",
                                        "icy-notice2:",
                                        "icy-name:",
                                        "icy-genre:",
                                        "icy-url:",
                                        "content-type:",
                                        "icy-pub:",
                                        "icy-metaint:",
                                        "icy-br:"
                                    };

class icy {
    public:

        // icy header/metadata
        char            icy_notice1[255]; // notice 1
        char            icy_notice2[255]; // notice 2
        char            icy_name[255]; // station name
        char            icy_genre[255]; // station genre
        char            icy_url[255]; // url
        char            content_type[255]; // type
        int             icy_pub; // public
        unsigned int    icy_metaint; // metaint .. very important
        int             icy_br; // bitrate
        char            stream_title[50]; // stream title
        char            stream_url[255]; // stream url
        char            track_title[50]; // track info

        // vars
        unsigned long   metaint_pos;
        bool            looking_for_header;
        unsigned long   buffered;
        unsigned long   pre_buffer; // buffer before playback ...
        unsigned long   buffer_size;
        char*           buffer;
        bool            bufferring;

        unsigned long   bytes_left_in_buffer;
        unsigned long   bytes_read_buffer;


    public:

    icy() : metaint_pos(0), looking_for_header(true), buffered(0), pre_buffer(50000), buffer_size(1000000),
            bufferring(true), bytes_left_in_buffer(0),bytes_read_buffer(0)
    {
        memset(track_title,0,50);
        buffer = new char[buffer_size];
        memset(buffer,0,buffer_size);

    };

    ~icy()
    {
        delete [] buffer;
        buffer = 0;
    };

    void icy_reset()
    {
        memset(buffer,0,buffer_size);
        memset(track_title,0,50);
        metaint_pos = 0;
        looking_for_header = true;
        buffered = 0;
        bufferring = true;
    }


    void parse_header_item(const char* obj)
    {
        char* start = 0;
        char* end = 0;
        char tmp[10] = {0};

        for(int i=0;i<ICY_META_MAX;i++)
        {
            char* a = icy_meta[i];
            start =  strstr(obj,icy_meta[i]);
            if (start) {
                // find the end of this meta data
                end = strstr(start,"\r\n");
                if (end) {
                    start += strlen(icy_meta[i]);
                    switch(i){
                        case 0: memcpy(icy_notice1,start,end-start); break;
                        case 1: memcpy(icy_notice2,start,end-start); break;
                        case 2: memcpy(icy_name,start,end-start); break;
                        case 3: memcpy(icy_genre,start,end-start); break;
                        case 4: memcpy(icy_url,start,end-start); break;
                        case 5: memcpy(content_type,start,end-start); break;
                        case 6:
                            memcpy(tmp,start,end-start);
                            icy_pub = atoi(tmp);
                            memset(tmp,0,10);
                         break;
                        case 7:
                            memcpy(tmp,start,end-start);
                            icy_metaint = atoi(tmp);
                            memset(tmp,0,10);
                        break;
                        case 8:
                            memcpy(tmp,start,end-start);
                            icy_br = atoi(tmp);
                            memset(tmp,0,10);
                        break;


                    }

                }
            }
        }

    };

    int parse_header(char* buf,int len)
    {
        char* found = strstr(buf,"ICY 200 OK"); // expect this response from the server !
        if (!found) return 0;

        parse_header_item(buf);

        //find the end of the header then remove the header from the stream.
        char* end_header = strstr(buf,"\r\n\r\n");
        if (end_header)
        {
            return (end_header + 4) - buf;
        }

        return 0;
    };

    void parse_meta_data(char* data,int len)
    {
        char* title_start = 0;
        char* title_end = 0;

        title_start = strstr(data,"StreamTitle='");
        if (title_start) {
        title_start += strlen("StreamTitle='");
        title_end = strstr(title_start,"';");
        if(title_end) {
            memset(track_title,0,50);
            memcpy(track_title,title_start,(title_end-title_start)>50 ? 50 : (title_end-title_start));
        }

        }
     };

    int parse_metaint(char* net_buffer,int len)
    {
        metaint_pos += len;

        if (looking_for_header) return len;

        int new_len = len;

        if (metaint_pos  > icy_metaint)
        {
            int diff = (metaint_pos - icy_metaint) ;
            int start_pos = len-diff;
            //metaint

            unsigned int metadata_length = (unsigned int)((unsigned char)net_buffer[start_pos]);
            if (metadata_length > 0) //sometimes is zero
            {
                int real_metadata_length = metadata_length*16; // meta len is stored / 16 to only take up 1 byte

                parse_meta_data(net_buffer+start_pos+1,real_metadata_length); // add 1 to bypass the len byte

                metaint_pos = diff - (real_metadata_length + 1) ; // calculate how many bytes of audio data are still in the buffer

                // remove the metadata from the stream !!!
                memcpy(net_buffer+start_pos,net_buffer+start_pos+1+real_metadata_length,len-start_pos-1-real_metadata_length);
                new_len -= real_metadata_length + 1; // adjust the len
            }else{

                metaint_pos = diff - 1; //fix the metapos
                // remove the zero byte from the stream
                memcpy(net_buffer+start_pos,net_buffer+start_pos+1,len-start_pos-1);
                new_len -= 1;
            }

        }
        return new_len;
    };

    void buffer_data(char* net_buffer,int len)
    {
        unsigned long new_buffer_size = buffered + len;

        if (new_buffer_size > (buffer_size-1))
        {
            long remain = new_buffer_size - (buffer_size-1);
            int end_bytes = (buffer_size-1) - buffered;

            // copy to the end of the buffer
            if (end_bytes > 0) memcpy(buffer+buffered,net_buffer,end_bytes);

            remain = len-end_bytes;

            // copy to the start of the buffer
            if (remain > 0 ) {
                memcpy(buffer,net_buffer+end_bytes,remain);
                buffered = remain;
            }else buffered = 0;

        }else{
            // keep copying to the buffer
            memcpy(buffer+buffered,net_buffer,len);
            buffered += len;
        }

        /*  state on fist connected ... needed to parse out the header.
            512 is a number I selected to ensure that there is enough data in the buffer
            to check for a full header.
        */
        if (looking_for_header && buffered > 512)
        {
            int remove = parse_header(buffer,buffered);
            if (remove)
            {
                memcpy(buffer,buffer+remove,buffered+remove);
                buffered -= remove;
                metaint_pos-=remove;

            }
            looking_for_header = false;
        }

        if (buffered >= pre_buffer) bufferring = false; // got enough data, change the state to allow playback


    };


};

#endif //_ICY_H_

